%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% ENGO 638 - August 2012                                                  %
% Lab No. 2                                                               %
% Daniele Borio                                                           %
% Minimal Tracking Loop                                                   %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

clc;
% open the input file
fid = fopen('../../GpsL116bitCpx_1575M5M_01Aug10.bin', 'r' );

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                       Signal Characteristics                            %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

fs = 5e6;       % Sampling frequency
fif = 0.42e6;    % Intermediate frequency
IsComplex = true;  % Tell if the samples are in I/Q format
Is16Bits  = true;  % Tell if the samples are stored on 16 bits/byte

fc = 1.023e6;   % code rate
codeLength = 1023;

% prn and initial values from acquisition
prn = 28;
Doppler = -890; % Doppler frequency in Hz from acquisition
codeDelay = 973.6914;   % Code delay in samples (this is actually 1023 * fs / fc - codeDelay)

bytesToSkip = round( ( 1023 - codeDelay ) * fs / fc );
if IsComplex,
    bytesToSkip = 2 * bytesToSkip;
end

if Is16Bits,
    bytesToSkip = 2 * bytesToSkip;
end
    
fseek(fid, bytesToSkip, 'bof'); % skip "codeDelay" samples to align the beginning of the code


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                Initialize some useful variables                         %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


numEpochs = 4000;          % number of epoch to process
sizeCircBuffers = 1000;     % size of the circular buffers used for storing the different quatities

% create different circular buffer to store the different quantities

% correlator outputs
prompt = struct('buffer', zeros(1, sizeCircBuffers), 'index', 1);
early = struct('buffer', zeros(1, sizeCircBuffers), 'index', 1);
late = struct('buffer', zeros(1, sizeCircBuffers), 'index', 1);

% Accumulators
Tc = 0.001;  % coherent integration time 1 ms

Nacc = 0;
Pacc = 0;
Eacc = 0;
Lacc = 0;

% discriminator outputs
codeDiscr = struct('buffer', zeros(1, sizeCircBuffers), 'index', 1);
carrDiscr = struct('buffer', zeros(1, sizeCircBuffers), 'index', 1);

% loop filter outputs
codeFilt = struct('buffer', zeros(1, sizeCircBuffers), 'index', 1);
carrFilt = struct('buffer', zeros(1, sizeCircBuffers), 'index', 1);

carrFilt.buffer( 1 ) = Doppler;

% C/N0 estimates and other indicators
cnoValue = struct('buffer', zeros(1, sizeCircBuffers), 'index', 1);
                        
% Loop filter parameters
carrBandwidth = 5;     % carrier loop bandwidth
codeBandwidth = 5;      % code loop bandwidth

carrOrder = 3;
codeOrder = 2;

% Set the loop filter parameters
carrLoopFilter = SetLoopCoefficients( carrBandwidth, carrOrder, Tc, Doppler );
codeLoopFilter = SetLoopCoefficients( codeBandwidth, codeOrder, Tc, Doppler / 1540 );

% get a vector with the C/A code sampled 1x/chip
ca = cacode(prn);

% early-minus-late offset
ds = 1;

% loop status
isBitSynched = false;  % initially we are not bit synched

% code and carrier phase reminder
remcodephase = 0;
remcarrphase = 0;

% create the bitsyncher
% HS = histogramSyncher;

HS = struct('bins', zeros(1, 20), 'currentBin', 0, 'previousPrompt', 0, 'Tlow', 3, 'Tup', 10, 'Synched', false );
skipTransient = 2000; % we don't try to bit synch until we have pulled in

% create the CNo estimator
% CNoEstimator = struct('complexCorr', zeros(1, 20), 'index', 1, 'alpha', 0.8, 'mu', 0, 'cno', 0);

% file for storing the results
outputname = sprintf('myTracking-PRN-%d.TRK', prn);
output = fopen( outputname, 'w');

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                      main processing loop                               %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

for ii = 1:numEpochs,

    if rem(ii, sizeCircBuffers) == 0,
        ii
        
        % write the data to file
        writeToDisk( output, prompt, early, late, carrDiscr, carrFilt, cnoValue );
    end

    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %            Determine the block size and read the data               %
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    codephasestep = (fc + codeFilt.buffer( codeFilt.index)) / fs;   % code step
    blksize = ceil((codeLength-remcodephase)/codephasestep);
    
    if( IsComplex ),
        if( Is16Bits ),
            [rawdata, cnt_data] = fread(fid, 2 * blksize, 'int16');
        else
            [rawdata, cnt_data] = fread(fid, 2 * blksize, 'int8');
        end
        rawdata = rawdata(1:2:end) + 1i * rawdata(2:2:end);
    else
        if( Is16Bits ),
            [rawdata, cnt_data] = fread(fid, blksize, 'int16');
        else
            [rawdata, cnt_data] = fread(fid, blksize, 'int8');
        end
    end
    rawdata = rawdata.';
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %            NCOs - generates local code and carrier                  %
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    % define index into Early code vector
    tcode = (remcodephase + ds/2):codephasestep:((blksize-1)*codephasestep + remcodephase + ds/2);
    tcode2 = rem(ceil(tcode + codeLength), codeLength) + 1;  
    earlycode = ca(tcode2);
    
    % define index into Late code vector
    tcode=(remcodephase - ds/2):codephasestep:((blksize-1)*codephasestep + remcodephase - ds/2);
    tcode2 = rem(ceil(tcode + codeLength), codeLength) + 1;  
    latecode = ca(tcode2);
    
    % define index into Prompt code vector
    tcode = remcodephase:codephasestep:((blksize-1)*codephasestep+remcodephase);
    tcode2 = rem(ceil(tcode + codeLength), codeLength) + 1;  
    promptcode = ca(tcode2);
    
    remcodephase = rem( blksize*codephasestep + remcodephase, codeLength);
    
    % generate the local carrier
    localcarrier = exp(-j*2*pi*(fif + carrFilt.buffer(codeFilt.index))*(0:(blksize - 1)) / fs - j*remcarrphase);
    remcarrphase = rem(2*pi*(fif + carrFilt.buffer(codeFilt.index))*blksize / fs + remcarrphase, 2 * pi);
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %                       Integrate & Dump                              %
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%    
    
    basebandsignal = rawdata.*localcarrier;

    prompt.index = rem( prompt.index, sizeCircBuffers ) + 1;
    early.index = rem( early.index, sizeCircBuffers ) + 1;
    late.index = rem( late.index, sizeCircBuffers ) + 1;
    
    prompt.buffer(prompt.index) = basebandsignal * promptcode.';
    early.buffer(early.index) = basebandsignal * earlycode.';
    late.buffer(late.index) = basebandsignal * latecode.';
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %             Accumulation - depending on bit synchronization         %
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    if( isBitSynched )
        
        % Accumulate for 20 ms
        Nacc = Nacc + 1;
        Pacc = Pacc + prompt.buffer(prompt.index);
        Eacc = Eacc + early.buffer(early.index);
        Lacc = Lacc + late.buffer(late.index);
        
        % Update the CN0 only if bit-synched
        % CNoEstimator = UpdateCNoEstimator( CNoEstimator, prompt.buffer(prompt.index));
        
        % if the coherent integration time is 20, update the loop
        if( Nacc == 20 )
            
            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
            %             Evaluate the discriminator output                       %
            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
            codeDiscr.index = rem( codeDiscr.index, sizeCircBuffers ) + 1;
            codeDiscr.buffer(codeDiscr.index) = UpdateCodeDiscriminator( Pacc, Eacc, Lacc, ds );

            carrDiscr.index = rem( carrDiscr.index, sizeCircBuffers) + 1;
            carrDiscr.buffer(carrDiscr.index) = UpdateCarrDiscriminator( Pacc );
            
            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
            %                   Evaluate the filter output                        %
            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
            codeFilt.index = rem( codeFilt.index, sizeCircBuffers ) + 1;
            [ codeFilt.buffer( codeFilt.index ) codeLoopFilter ] = ...
                LoopFilter( codeLoopFilter, codeDiscr.buffer(codeDiscr.index) );

            carrFilt.index = rem( carrFilt.index, sizeCircBuffers ) + 1;
            [ carrFilt.buffer( carrFilt.index ) carrLoopFilter ] = ...
                LoopFilter( carrLoopFilter, carrDiscr.buffer(carrDiscr.index) );
        
            % Reset the accumulators
            Pacc = 0;
            Eacc = 0;
            Lacc = 0;
            Nacc = 0;
            
        else % propagate the estimates
            
            % propagate the estimates of the discriminator outputs
            previousValue = codeDiscr.buffer( codeDiscr.index );                % record the previous value
            codeDiscr.index = rem( codeDiscr.index, sizeCircBuffers ) + 1;      % update the index
            codeDiscr.buffer( codeDiscr.index ) = previousValue;                % propagate the previous value
            
            previousValue = carrDiscr.buffer( carrDiscr.index );                % record the previous value
            carrDiscr.index = rem( carrDiscr.index, sizeCircBuffers ) + 1;      % update the index
            carrDiscr.buffer( carrDiscr.index ) = previousValue;                % propagate the previous value
            
            % propagate the estimates of the filter outputs
            previousValue = codeFilt.buffer( codeFilt.index );                % record the previous value
            codeFilt.index = rem( codeFilt.index, sizeCircBuffers ) + 1;      % update the index
            codeFilt.buffer( codeFilt.index ) = previousValue;                % propagate the previous value
            
            previousValue = carrFilt.buffer( carrFilt.index );                % record the previous value
            carrFilt.index = rem( carrFilt.index, sizeCircBuffers ) + 1;      % update the index
            carrFilt.buffer( carrFilt.index ) = previousValue;                % propagate the previous value
                        
        end
    else
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %             Evaluate the discriminator output                       %
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        codeDiscr.index = rem( codeDiscr.index, sizeCircBuffers ) + 1;
        codeDiscr.buffer(codeDiscr.index) = ...
                UpdateCodeDiscriminator( prompt.buffer(prompt.index), ...
                                         early.buffer(early.index), ...
                                         late.buffer(late.index), ds );
                                               
        carrDiscr.index = rem( carrDiscr.index, sizeCircBuffers) + 1;
        carrDiscr.buffer(carrDiscr.index) = ...
                    UpdateCarrDiscriminator( prompt.buffer(prompt.index) );
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %                   Evaluate the filter output                        %
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%   
        codeFilt.index = rem( codeFilt.index, sizeCircBuffers ) + 1;
        [ codeFilt.buffer( codeFilt.index ) codeLoopFilter ] = ...
            LoopFilter( codeLoopFilter, codeDiscr.buffer(codeDiscr.index) );
        
        carrFilt.index = rem( carrFilt.index, sizeCircBuffers ) + 1;
        [ carrFilt.buffer( carrFilt.index ) carrLoopFilter ] = ...
            LoopFilter( carrLoopFilter, carrDiscr.buffer(carrDiscr.index) );
        
    end
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %                          C/N0 estimator                             %
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % cnoValue.index = rem( cnoValue.index, sizeCircBuffers ) + 1;
    % cnoValue.buffer( cnoValue.index ) = CNoEstimator.cno;
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %                               bit synch                             %
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    if ( ii > skipTransient ),
        % Update the bit-syncher using the prompt accumulator
	% HS = AddSymbol( HS, prompt.buffer( prompt.index ) );
        if ~isBitSynched && HS.Synched
            % bar( HS.bins )
            % grid on
            % ii
            isBitSynched = true;

            % initialize the accumualtion
            Pacc = prompt.buffer( prompt.index );
            Eacc = early.buffer( early.index );
            Lacc = late.buffer( late.index );

            NAcc = 1;

            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
            %                       Update the loop filters               %
            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

            carrLoopFilter = SetLoopCoefficients( carrBandwidth, carrOrder, ...
                                20 * Tc, carrFilt.buffer(carrFilt.index) );
            codeLoopFilter = SetLoopCoefficients( codeBandwidth, codeOrder, ...
                                20 * Tc, codeFilt.buffer(codeFilt.index) );
                            
            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
            %                       Start C/N0 estimation                 %
            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%        
            % CNoEstimator = UpdateCNoEstimator( CNoEstimator, prompt.buffer(prompt.index));                
        end
    end

    
end

% close the input and output files
fclose( fid );
fclose( output );

%
plotResults
