clear all
close all
clc

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                 Specify the PRN and the update rate                     %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

prn = 28;
Tc = 0.001;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                 Load the data and build the time scale                  %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

y = load( [ 'myTracking-PRN-' num2str( prn ) '.TRK' ] );
Nepoch = length(y);
tt = (1:Nepoch) * Tc;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                 Square Modulus of the correlator Output                 %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
figure
subplot( 2, 3, 1)
plot( tt, y(:,1).^2 + y(:,2).^2, ...    % Prompt
      tt, y(:,3).^2 + y(:,4).^2, ...    % Early
      tt, y(:,5).^2 + y(:,6).^2 )       % Late
grid on
box on
set( gca, 'FontSize', 14 );
  
xlabel( 'Time [s]' )
ylabel( 'Corr. Output' );
legend( 'P', 'E', 'L', 'Location', 'NorthEast' );

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                 I/Q components of the Prompt Correlator                 %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
subplot(2, 3, 4);
plot(tt, y(:,1), tt, y(:,2))

grid on
box on
set( gca, 'FontSize', 14 );

xlabel( 'Time [s]' );
ylabel( 'Prompt Corr. Output' );
legend( 'I', 'Q', 'Location', 'NorthEast' );

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                 C/N0 estimates                                          %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

subplot( 2, 3, 2 )
plot( tt, y(:,9) )

grid on
box on
set( gca, 'FontSize', 14 );

xlabel( 'Time [s]' );
ylabel( 'C/N_0 [dB-Hz]' );

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                 Carrier and Code Doppler                                %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

subplot(2,3,3), plot(tt, y(:, 8));
grid on
box on
set( gca, 'FontSize', 14 );
ylabel( 'Carrier Doppler [Hz]' );
xlabel( 'Time [s]' );

subplot(2,3,6), plot(tt, y(:, 7));
grid on
box on
set( gca, 'FontSize', 14 );
ylabel( 'Code Disc [chips]' );
xlabel( 'Time [s]' );

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                           Scatter Plot                                  %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

subplot( 2, 3, 5 );
plot( y(:,1), y(:,2), 'b.' );
grid on
box on
set( gca, 'FontSize', 14 );
xlabel( 'I' );
ylabel( 'Q' );
axis equal